import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import styled from 'styled-components';

import { Button } from '../styles';


const Container = styled.form`
  padding: 1rem 0;
  display: flex;
  flex-flow: row wrap;
`;

const Legend = styled.h3`
  flex: 1 0 100%;
  margin: 0.5rem 0;
`;

const Field = styled.label`
  display: flex;
  flex-flow: column nowrap;
  align-items: start;

  position: relative;
  margin: 0 1rem 1rem 0;
  padding-right: 1rem;
  flex: 1 1 auto;

  ${props => props.wide && `
    flex: 1 1 100%;
  `}

  ${props => props.required && `
    ::after {
      content: '*';
      color: #c44;
      position: absolute;
      right: 0;
    }
  `}
`;

const Label = styled.span`
  font-weight: 500;

  ::after {
    content: ':';
  }
`;

const Input = styled.input`
  width: 100%;
  border: 1px solid #666;
  border-radius: 0.5rem;
  padding: 0.1rem 0.5rem;
`;

const Textarea = styled(Input).attrs({
  as: "textarea",
})`
  resize: vertical;
  min-height: 5rem;
`;

const Submit = styled(Button).attrs({
  type: "submit",
  as: "input",
})`
  margin: 0;
`;


export default class Form extends PureComponent {

  constructor(props) {
    super(props);

    this.form = React.createRef();

    this.content = React.createRef();
    this.email = React.createRef();
    this.name = React.createRef();
    this.url = React.createRef();
  }
  
  static propTypes = {
    createNew: PropTypes.func,
    replyTo: PropTypes.number,
    cancelReply: PropTypes.func,
  };

  scrollTo() {
    this.form.current.scrollIntoView();
  }

  submitHandler = (e) => {
    e.preventDefault();

    const values = {
      email: this.email.current.value,
      name: this.name.current.value,
      url: this.url.current.value,
      content: this.content.current.value,
      parent: this.props.replyTo,
    };

    this.props.createNew(values);

    this.content.current.value = '';

    this.props.cancelReply();

    return false;
  }

  render() {
    const { email, name, url, replyTo, cancelReply } = this.props;

    return (
      <Container onSubmit={this.submitHandler} ref={this.form}>
        {replyTo ?
          <Legend>
            Post a reply to #{replyTo}
            <Button onClick={cancelReply} inline>Cancel</Button>
          </Legend>
        :
          <Legend>Post a comment</Legend>
        }

        <Field required>
          <Label>Email</Label>
          <Input type="email" name="email" required ref={this.email} defaultValue={email} />
        </Field>

        <Field required>
          <Label>Name</Label>
          <Input type="text" name="name" required ref={this.name} defaultValue={name} />
        </Field>

        <Field>
          <Label>Website</Label>
          <Input type="url" name="url" ref={this.url} defaultValue={url} />
        </Field>

        <Field required wide>
          <Label>Content</Label>
          <Textarea name="content" required ref={this.content} />
        </Field>

        <Submit value="Submit" />
      </Container>
    );
  }
}
