import React, { PureComponent } from 'react';
import PropTypes from 'prop-types';
import { Link } from 'react-router-dom';
import styled from 'styled-components';

import NotFound from '../NotFound';
import { TextPlaceholder, GradientBorder } from '../styles';
import { DateTime, Icon } from '../_global';

import { UserDetail } from '../users';
import { CommentsList } from '../comments';
import { CategoryDetail, TagDetail } from '../categories';



const Article = styled.article`
  padding: 2rem;
  margin: 1rem 0;
`;

const Header = styled.header`
  padding: 1rem 0 0.5rem 0;
  margin-bottom: 2rem;
  ${GradientBorder()}
  border-width: 2px 0;
`;

const Title = styled.h2`
  font-size: 2rem;
  margin: 0;
  
  a {
    color: inherit;

    :hover, :active, :focus {
      text-decoration: none;
      color: #666;
    }
  }
`;

const HeaderPart = styled.span`
  display: inline-block;
  padding: 0.5rem 1.5rem 0.5rem 0;
  color: #666;
  font-weight: 600;
`;

const Body = styled.main`
  hr {
    ${GradientBorder()}
    border-bottom-width: 2px;
  }

  h2, h3, h4 {
    margin-bottom: 0;
  }

  p {
    margin-top: 0;
  }
`;

const Footer = styled.footer`
  ${GradientBorder()}
  border-top-width: 2px;
  padding: 0.5rem 0;
`;


export default class Post extends PureComponent {
  
  static propTypes = {
    full: PropTypes.bool,

    //OPT: import {PostShape} from './propTypes'; post: PostShape,
    post: PropTypes.object,
    link: PropTypes.oneOfType([
      PropTypes.string,
      PropTypes.shape({
        pathname: PropTypes.string,
        state: PropTypes.string,
      }),
    ]),
  };

  renderPlaceholder() {
    return (
      <Article>
        <Header>
          <Title>
            <TextPlaceholder />
          </Title>

          <HeaderPart>
            <UserDetail />
          </HeaderPart>
          <HeaderPart>
            <TextPlaceholder />
          </HeaderPart>
        </Header>

        <Body>
          <p>
            <TextPlaceholder />
            <TextPlaceholder />
            <TextPlaceholder />
          </p>
          <p>
            <TextPlaceholder />
            <TextPlaceholder />
          </p>
        </Body>

        <Footer>
          <TextPlaceholder />
        </Footer>
      </Article>
    );
  }

  renderExcerpt({id, date, title, excerpt, author}, link) {
    return (
      <Article>
        <Header>
          <Title><Link to={link}>
            {title.rendered}
          </Link></Title>

          <HeaderPart>
            by&nbsp;
            <UserDetail embed id={author} />
          </HeaderPart>
          <HeaderPart>
            <Icon name="date" />
            <DateTime date={date} />
          </HeaderPart>
        </Header>

        <Body dangerouslySetInnerHTML={{__html: excerpt.rendered}} />

        <Footer>
          <Link to={link}>Read more</Link>
        </Footer>
      </Article>
    );
  }

  renderFull({id, date, modified, title, excerpt, content, author, categories, tags}, link) {
    return (
      <Article>
        <Header>
          <Title>{title.rendered}</Title>

          <HeaderPart>
            by&nbsp;
            <UserDetail embed id={author} />
          </HeaderPart>
          <HeaderPart>
            <Icon name="date" />
            <DateTime date={date} modified={modified} />
          </HeaderPart>
        </Header>

        {/* when coming from posts list, excerpt is loaded, content isn't, and the first p of text is a very good placeholder for the full text */}
        <Body dangerouslySetInnerHTML={{__html: content ? content.rendered : excerpt.rendered}} />

        <Footer>
          {link && <HeaderPart><Link to={link}>Back</Link></HeaderPart>}
          {categories && categories.length > 0 &&
            <HeaderPart>
              {categories.map(c => 
                <CategoryDetail key={c} embed id={c} />
              )}
            </HeaderPart>
          }
          {tags && tags.length > 0 &&
            <HeaderPart>
              {tags.map(t => 
                <TagDetail key={t} embed id={t} />
              )}
            </HeaderPart>
          }
        </Footer>

        <CommentsList as="aside" />
      </Article>
    );
  }

  render() {
    const {post, link, full} = this.props;

    if (!post) {
      return this.renderPlaceholder();
    } else if (post === 404) {
      return <NotFound />;
    } else if (!full) {
      return this.renderExcerpt(post, link);
    } else {
      return this.renderFull(post, link);
    }
  }
}
